/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        24.10.2007
 Description:    Webserver uvm.

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
----------------------------------------------------------------------------*/

#include <util/delay.h>
#include <avr/io.h>
#include "timer.h"
#include "dmx.h"

//----------------------------------------------------------------------------
//Systemtakt einstellen
void clock_init (void)
{
	//Achtung F_CPU = 32000000UL Setting in Toolchain / Symbols beachten
	
	// Taktsystem konfigurieren
	// externen 8MHz Quarz einschalten
	OSC.XOSCCTRL = OSC_XOSCSEL_XTAL_16KCLK_gc | OSC_FRQRANGE_2TO9_gc;
	OSC.CTRL |= OSC_XOSCEN_bm;
	while(!(OSC.STATUS & OSC_XOSCRDY_bm));
	
	//PLL 8MHz * 4 = 32MHz
	OSC.PLLCTRL = OSC_PLLSRC_XOSC_gc | 0x04;
	//PLL einschalten
	OSC.CTRL |= OSC_PLLEN_bm;
	while (!(OSC.STATUS & OSC_PLLRDY_bm));
		
	//IO Register schtzen / Interrupts sperren
	CCP = CCP_IOREG_gc;
	//Externe Taktquelle auswhlen
	//CLK.CTRL = CLK_SCLKSEL_XOSC_gc; //8MHZ ohne PLL
	CLK.CTRL = CLK_SCLKSEL_PLL_gc; //32MHz mit PLL
}

//----------------------------------------------------------------------------
//Hier startet das Hauptprogramm
int main(void)
{  	
	uint16_t a;
	uint8_t sw;
	
	//PULLUP Switch 1-5
	PORTA.DIRCLR |= PIN0_bm|PIN1_bm|PIN2_bm|PIN3_bm|PIN4_bm;
	//Pullup
	PORTA.PIN0CTRL |= PORT_OPC_PULLUP_gc;
	PORTA.PIN1CTRL |= PORT_OPC_PULLUP_gc;
	PORTA.PIN2CTRL |= PORT_OPC_PULLUP_gc;
	PORTA.PIN3CTRL |= PORT_OPC_PULLUP_gc;
	PORTA.PIN4CTRL |= PORT_OPC_PULLUP_gc;
	
	//Systemtakt einstellen
	clock_init();
		
	//LED1 an Port PB0 PINB0 auf Ausgang
	PORTB.DIR |= PIN0_bm;
	
	//Applikationen starten
	dmx_init();
	timer_init();

	//Globale Interrupts einschalten
	sei(); 
	
	while(1)
	{
		sw = ((~PORTA.IN) & 0x0F);	//Switch 1 - 4 in %
		if(sw > 10) sw = 10;
		
		for(a = 0;a<513;a++)
		{	
			if(dmxUniverse1[a] > dmxUniverse2[a])
			{	
				dmxUniverse4[a] = dmxUniverse1[a] * sw / 10;
			}
			else
			{
				dmxUniverse4[a] = dmxUniverse2[a] * sw / 10;
			}			    
		}
    }	
		
return(0);
}

