/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        24.10.2007
 Description:    DMX Manipulator
----------------------------------------------------------------------------*/

#include <util/delay.h>
#include <avr/io.h>
#include "timer.h"
#include "dmx.h"

//----------------------------------------------------------------------------
//Systemtakt einstellen
void clock_init (void)
{
	//Achtung F_CPU = 32000000UL Setting in Toolchain / Symbols beachten
	
	// Taktsystem konfigurieren
	// externen 8MHz Quarz einschalten
	OSC.XOSCCTRL = OSC_XOSCSEL_XTAL_16KCLK_gc | OSC_FRQRANGE_2TO9_gc;
	OSC.CTRL |= OSC_XOSCEN_bm;
	while(!(OSC.STATUS & OSC_XOSCRDY_bm));
	
	//PLL 8MHz * 4 = 32MHz
	OSC.PLLCTRL = OSC_PLLSRC_XOSC_gc | 0x04;
	//PLL einschalten
	OSC.CTRL |= OSC_PLLEN_bm;
	while (!(OSC.STATUS & OSC_PLLRDY_bm));
		
	//IO Register schtzen / Interrupts sperren
	CCP = CCP_IOREG_gc;
	//Externe Taktquelle auswitchhlen
	//CLK.CTRL = CLK_SCLKSEL_XOSC_gc; //8MHZ ohne PLL
	CLK.CTRL = CLK_SCLKSEL_PLL_gc; //32MHz mit PLL
}

//----------------------------------------------------------------------------
//Hier startet das Hauptprogramm
int main(void)
{  	
	uint16_t a;
	uint16_t b;
	
	uint8_t schalter,value1,value2;
	uint8_t tmp;
	
	//PULLUP DMX IN/OUT Switch1-4
	//Switch 1 for for USART1
	//Switch 2 - 4 spare
	//Port Input
	PORTA.DIRCLR |= PIN0_bm|PIN1_bm|PIN2_bm|PIN3_bm;
	//Pullup
	PORTA.PIN0CTRL |= PORT_OPC_PULLUP_gc;
	PORTA.PIN1CTRL |= PORT_OPC_PULLUP_gc;
	PORTA.PIN2CTRL |= PORT_OPC_PULLUP_gc;
	PORTA.PIN3CTRL |= PORT_OPC_PULLUP_gc;
	
	//Systemtakt einstellen
	clock_init();
		
	//LED1 an Port PB0 PINB0 auf Ausgang
	PORTB.DIR |= PIN0_bm;
	
	//Applikationen starten
	dmx_init();
	timer_init();

	//Globale Interrupts einschalten
	sei(); 
	
	while(1)
	{
		schalter = ((~PORTA.IN) & 0x0F);	
		if(schalter > 10) schalter = 10;
		
		switch (schalter)
		{
			case 1:
				value1 = 20;
				value2 = 10;
				break;
			
			case 2:
				value1 = 40;
				value2 = 20;
				break;
			
			case 3:
				value1 = 60;
				value2 = 30;
				break;
			
			case 4:
				value1 = 80;
				value2 = 40;
				break;
			
			case 5:
				value1 = 100;
				value2 = 50;
				break;
			
			case 6:
				value1 = 100;
				value2 = 40;
				break;
			
			case 7:
				value1 = 100;
				value2 = 30;
				break;
			
			case 8:
				value1 = 100;
				value2 = 20;
				break;
			
			case 9:
				value1 = 100;
				value2 = 10;
				break;
			
			case 10:
				value1 = 100;
				value2 = 0;
				break;
			
			default:
				value1 = 100;
				value2 = 0;
				break;
		}
	
		b = 0;
		for(a = 0;a<513;a+=2)
		{		
				//gerade Kanle 0,2,4...........
				tmp = ((float)(dmxUniverse1[b]*value1/100)) - ( ((float)(dmxUniverse1[b]*value2/100)) * ((float)dmxUniverse1[b+1]/255) );
				dmxUniverse4[b] = tmp;
				b++;
				
				//ungerade Kanle 1,3,5..........
				tmp = ((float)(dmxUniverse1[b]*value1/100)) - ( ((float)(dmxUniverse1[b]*value2/100)) * ((float)dmxUniverse1[b-1]/255) );
				dmxUniverse4[b] = tmp;
				b++;		    
		}
    }	
		
return(0);
}

